// Adaptovano z generatoru pro Kasiopeu - ulohu F z finale 2018/19
#include <iostream>
#include <vector>
#include <cmath>
#include <algorithm>
#include <string>
#include <set>
#include <map>
#include <unordered_set>
#include <unordered_map>
#include <queue>
#include <random>
#include <stack>
#include <cstring>
#include <cassert>
#include <fstream>
using namespace std;
using ll = int64_t;
#define rep(i,a,n) for (int i=a;i<(int)(n);i++)
#define per(i,a,n) for (int i=(n)-1;i>=(int)(a);i--)
template<typename T> ostream& operator<<(ostream& _s, vector<T> _t) {
    rep(i, 0, _t.size()) _s << (i ? " " : "") << _t[i]; return _s;
}
template<typename T> istream& operator>>(istream& _s, vector<T> &_t) {
    rep(i, 0, _t.size()) _s >> _t[i]; return _s;
}
template<typename T, typename U> ostream& operator<<(ostream& _s, pair<T, U> _t) {
    _s << "(" << _t.first << "," << _t.second << ")"; return _s;
}
template<typename T, typename U> istream& operator>>(istream& _s, pair<T, U> &_t) {
    _s >> _t.first >> _t.second; return _s;
}

uint64_t random64() {
    return (((uint64_t)rand() << 0) & 0x000000000000FFFFULL) |
           (((uint64_t)rand() << 16) & 0x00000000FFFF0000ULL) |
           (((uint64_t)rand() << 32) & 0x0000FFFF00000000ULL) |
           (((uint64_t)rand() << 48) & 0xFFFF000000000000ULL);
}

uint32_t random32() {
    return (((uint32_t)rand() << 0) & 0x0000FFFF) |
           (((uint32_t)rand() << 16) & 0xFFFF0000);
}

double randDouble() {
    return (double)random32() / 0xFFFFFFFF;
}

int randAlmost(int n) {
    const double MIN = 0.95;
    const double coef = randDouble() * (1 - MIN) + MIN;
    const int res = int(coef * n);
    assert(1 <= res && res <= n);
    return res;
}

struct Edge {
    int a, b;
};

struct Graph {
    int n;
    vector<Edge> edges;

    Graph(int _n) : n(_n) {}

    vector<vector<int>> getAdjacencyList() const {
        vector<vector<int>> adj(n);
        for (const Edge& e : edges) {
            adj[e.a].push_back(e.b);
            adj[e.b].push_back(e.a);
        }
        return adj;
    }

    void checkConnected() const {
        auto adj = getAdjacencyList();
        vector<int> q = {0};
        vector<int> from(n, -1);
        from[0] = -2;
        rep(i, 0, n) {
            int a = q[i];
            for (int b : adj[a]) {
                if (from[b] != -1) {
                    assert(from[a] == b);
                } else {
                    from[b] = a;
                    q.push_back(b);
                }
            }
        }
    }

    map<int, int> getDegrees() const {
        vector<int> deg(n, 0);
        for (const Edge& e : edges) {
            deg[e.a]++;
            deg[e.b]++;
        }
        map<int, int> m;
        for (int x : deg) {
            m[x]++;
        }
        return m;
    }

    void check(int max_n, bool first_must_be_leaf, int set_i) const {
        assert(n >= 2 && n <= max_n);
        assert(int(edges.size()) == n - 1);
        for (const Edge& e : edges) {
            assert(0 <= e.a && e.a < n);
            assert(0 <= e.b && e.b < n);
        }
        checkConnected();
        if (first_must_be_leaf) {
            int cnt = 0;
            for (const Edge& e : edges) {
                cnt += (e.a == 0) + (e.b) == 0;
            }
            // assert(cnt == 1);
        }
        if (set_i < 3) {
            map<int, int> vertices_with_deg = getDegrees();
            if (set_i == 2) { // prave 1 vrchol se stupnem >2
                assert(vertices_with_deg[1] + vertices_with_deg[2] == n - 1);
            } else { // prvni dve sady - cesta
                assert(vertices_with_deg[1] + vertices_with_deg[2] == n);
            }
        }
    }

    pair<int, int> getFurthest(int a0, vector<vector<int>> adj) const {
        vector<int> q = {a0};
        vector<int> dist(n, -1);
        dist[a0] = 0;
        rep(i, 0, n) {
            int a = q[i];
            for (int b : adj[a]) {
                if (dist[b] != -1) continue;
                dist[b] = dist[a] + 1;
                q.push_back(b);
            }
        }
        return {q[n - 1], dist[q[n - 1]]};
    }

    int getLongestPath() const {
        auto adj = getAdjacencyList();
        return getFurthest(getFurthest(0, adj).first, adj).second;
    }

    void shuffleIndices(mt19937& rng) {
        // Zamichej indexy vrcholu i hran
        vector<int> p(n);
        rep(i, 0, n) {
            p[i] = i;
        }
        shuffle(p.begin(), p.end(), rng);
        for (Edge& e : edges) {
            e.a = p[e.a];
            e.b = p[e.b];
            if (rand() % 2) {
                swap(e.a, e.b);
            }
        }
        shuffle(edges.begin(), edges.end(), rng);
    }

    void makeFirstLeaf() {
        // Zaridi, aby vrchol 0 byl list
        vector<int> deg(n, 0);
        for (const Edge& e : edges) {
            deg[e.a]++;
            deg[e.b]++;
        }
        if (deg[0] == 1) return;
        int x = 0;
        rep(i, 0, n) {
            if (deg[i] == 1) {
                x = i;
                break;
            }
        }
        for (Edge& e : edges) {
            if (e.a == x) {
                e.a = 0;
            } else if (e.a == 0) {
                e.a = x;
            }
            if (e.b == x) {
                e.b = 0;
            } else if (e.b == 0) {
                e.b = x;
            }
        }
    }

    void print(char* filename) const {
        FILE* f = fopen(filename, "w");
        fprintf(f, "%d\n", n);

        auto adj = getAdjacencyList();
        vector<int> p(n, -1);
        vector<int> dfs_ord = {0};
        p[0] = 0;
        rep(i, 0, n) {
            int a = dfs_ord[i];
            for (int b : adj[a]) {
                if (p[b] == -1) {
                    dfs_ord.push_back(b);
                    p[b] = i;
                }
            }
            if (i > 0) {
                assert(0 <= p[a] && p[a] < i);
                fprintf(f, "%d\n", p[a] + 1);
            }
        }
        fclose(f);
    }
};

Graph genStar(int n, int arm_length) {
    n = n - n % arm_length - arm_length + 1;
    Graph res(n);
    rep(i, 0, n - 1) {
        if (i % arm_length == 0) {
            res.edges.push_back({0, i + 1});
        } else {
            res.edges.push_back({i, i + 1});
        }
    }
    return res;
}

Graph genIrregularStar(int n) {
    int n_arms = n / 25;
    set<int> arms;
    while (int(arms.size()) < n_arms) {
        int x = 1 + random32() % (n - 1);
        arms.insert(x);
    }

    Graph res(n);
    rep(i, 0, n - 1) {
        if (arms.count(i)) {
            res.edges.push_back({0, i + 1});
        } else {
            res.edges.push_back({i, i + 1});
        }
    }
    return res;
}

Graph genPath(int n) {
    Graph res(n);
    rep(i, 0, n - 1) {
        res.edges.push_back({i, i + 1});
    }
    return res;
}

Graph genShort(int n) {
    Graph res(n);
    rep(i, 0, n - 1) {
        res.edges.push_back({(int)(random32() % (i + 1)), i + 1});
    }
    return res;
}

Graph genLong(int n, double p) {
    Graph res(n);
    rep(i, 0, n - 1) {
        int to = i;
        while (to > 0 && randDouble() > p) {
            to--;
        }
        res.edges.push_back({to, i + 1});
    }
    return res;
}

Graph genSample1() {
    Graph res(5);
    res.edges = {{0, 1}, {1, 2}, {2, 3}, {1, 4}};
    return res;
}

Graph genMinCase() {
    Graph res(2);
    res.edges = {{1, 0}};
    return res;
}

const vector<int> max_ns = {
    100,
    1000000,
    1000000,
    100, 100,
    2000, 2000,
    1000000, 1000000, 1000000,
};

vector<vector<Graph>> genAll() {
    vector<vector<Graph>> res;

    // 1 - cesta
    res.push_back({genMinCase(), genPath(max_ns[0]), genPath(randAlmost(max_ns[0]))});
    // 2 - cesta
    res.push_back({genMinCase(), genPath(max_ns[1]), genPath(randAlmost(max_ns[1]))});
    // 3 - prave 1 vrchol se stupnem >2
    res.push_back({
        genSample1(),
        genStar(randAlmost(max_ns[2]), 1),
        genStar(randAlmost(max_ns[2]), 2),
        genIrregularStar(randAlmost(max_ns[2])),
        genIrregularStar(randAlmost(max_ns[2])),
    });
    // 4-10
    rep(set_i, 3, 10) {
        int max_n = max_ns[set_i];
        res.push_back({
            genSample1(),
            genMinCase(),
            genStar(randAlmost(max_n), 1),
            genIrregularStar(randAlmost(max_n)),
            genShort(randAlmost(max_n)),
            genLong(randAlmost(max_n), 0.7),
            genLong(randAlmost(max_n), 0.99),
            genPath(randAlmost(max_n)),
        });
    }
    return res;
}

int main(int argc, char * argv[]) {
    const int seed = 123;
    srand(seed);
    mt19937 rng(seed);

    vector<vector<Graph>> test_sets = genAll();

    rep(set_i, 0, 10) {
        int max_n = max_ns[set_i];
        vector<Graph>& tests = test_sets[set_i];
        printf("Sada %d\n", set_i + 1);
        rep(test_i, 0, tests.size()) {
            Graph& g = tests[test_i];

            if (test_i > 0) {
                g.shuffleIndices(rng);
            }
            bool first_must_be_leaf = test_i % 2;
            if (first_must_be_leaf) {
                g.makeFirstLeaf();
            }
            g.check(max_n, first_must_be_leaf, set_i);

            char filename[30];
            sprintf(filename, "tests/%02d.%c.in", set_i + 1, 'a' + test_i + 1);
            g.print(filename);
            printf("  i=%2d n=%8d hloubka=%8d\n", test_i + 1, g.n, g.getLongestPath());
        }
    }
}
